@@ -1,4 +1,4 @@
-;# $Id: Storable.pm,v 0.4.1.1 1997/01/22 14:17:19 ram Exp $
+;# $Id: Storable.pm,v 0.5.1.2 1998/01/13 16:51:10 ram Exp $
 ;#
 ;#  Copyright (c) 1995-1997, Raphael Manfredi
 ;#  
@@ -6,11 +6,15 @@
 ;#  as specified in the README file that comes with the distribution.
 ;#
 ;# $Log: Storable.pm,v $
-;# Revision 0.4.1.1  1997/01/22  14:17:19  ram
-;# patch1: documented new forgive_me variable
+;# Revision 0.5.1.2  1998/01/13  16:51:10  ram
+;# patch2: added binmode() calls for systems where it matters
+;# patch2: be sure to pass globs, not plain file strings, to C routines
 ;#
-;# Revision 0.4  1997/01/15  18:20:09  ram
-;# Baseline for fourth netwide alpha release.
+;# Revision 0.5.1.1  1997/11/05  09:47:42  ram
+;# patch1: updated version number
+;#
+;# Revision 0.5  1997/06/10  16:38:37  ram
+;# Baseline for fifth alpha release.
 ;#
 
 require DynaLoader;
@@ -18,11 +22,18 @@ require Exporter;
 package Storable; @ISA = qw(Exporter DynaLoader);
 
 @EXPORT = qw(store retrieve);
-@EXPORT_OK = qw(nstore store_fd nstore_fd retrieve_fd);
+@EXPORT_OK = qw(
+	nstore store_fd nstore_fd retrieve_fd
+	freeze nfreeze thaw
+	dclone
+);
 
 use AutoLoader;
 use Carp;
-use vars qw($forgive_me);
+use vars qw($forgive_me $VERSION);
+
+$VERSION = '0.5_02';
+*AUTOLOAD = \&AutoLoader::AUTOLOAD;		# Grrr...
 
 bootstrap Storable;
 1;
@@ -49,7 +60,7 @@ sub nstore {
 	return _store(1, @_);
 }
 
-# Internal store routine
+# Internal store to file routine
 sub _store {
 	my $netorder = shift;
 	my $self = shift;
@@ -58,9 +69,10 @@ sub _store {
 	croak "Too many arguments" unless @_ == 1;	# Watch out for @foo in arglist
 	local *FILE;
 	open(FILE, ">$file") || croak "Can't create $file: $!";
+	binmode FILE;				# Archaic systems...
 	my $ret;
 	# Call C routine nstore or pstore, depending on network order
-	eval { $ret = $netorder ? net_pstore(FILE, $self) : pstore(FILE, $self) };
+	eval { $ret = $netorder ? net_pstore(*FILE, $self) : pstore(*FILE, $self) };
 	close(FILE) or $ret = undef;
 	unlink($file) or warn "Can't unlink $file: $!\n" if $@ || !defined $ret;
 	croak $@ if $@ =~ s/\.?\n$/,/;
@@ -87,7 +99,7 @@ sub nstore_fd {
 	return _store_fd(1, @_);
 }
 
-# Internal store routine
+# Internal store routine on opened file descriptor
 sub _store_fd {
 	my $netorder = shift;
 	my $self = shift;
@@ -103,6 +115,37 @@ sub _store_fd {
 	return $ret ? $ret : undef;
 }
 
+#
+# freeze
+#
+# Store oject and its hierarchy in memory and return a scalar
+# containing the result.
+#
+sub freeze {
+	_freeze(0, @_);
+}
+
+#
+# nfreeze
+#
+# Same as freeze but in network order.
+#
+sub nfreeze {
+	_freeze(1, @_);
+}
+
+# Internal freeze routine
+sub _freeze {
+	my $netorder = shift;
+	my $self = shift;
+	croak "Not a reference" unless ref($self);
+	croak "Too many arguments" unless @_ == 0;	# Watch out for @foo in arglist
+	my $ret;
+	# Call C routine mstore or net_mstore, depending on network order
+	eval { $ret = $netorder ? net_mstore($self) : mstore($self) };
+	croak $@ if $@ =~ s/\.?\n$/,/;
+	return $ret ? $ret : undef;
+}
 #
 # retrieve
 #
@@ -113,8 +156,9 @@ sub retrieve {
 	my ($file) = @_;
 	local *FILE;
 	open(FILE, "$file") || croak "Can't open $file: $!";
+	binmode FILE;							# Archaic systems...
 	my $self;
-	eval { $self = pretrieve(FILE) };		# Call C routine
+	eval { $self = pretrieve(*FILE) };		# Call C routine
 	close(FILE);
 	croak $@ if $@ =~ s/\.?\n$/,/;
 	return $self;
@@ -135,6 +179,20 @@ sub retrieve_fd {
 	return $self;
 }
 
+#
+# thaw
+#
+# Recreate objects in memory from an existing frozen image created
+# by freeze.
+#
+sub thaw {
+	my ($frozen) = @_;
+	my $self;
+	eval { $self = mretrieve($frozen) };	# Call C routine
+	croak $@ if $@ =~ s/\.?\n$/,/;
+	return $self;
+}
+
 =head1 NAME
 
 Storable - persistency for perl data structures
@@ -191,6 +249,25 @@ retrieve does not provide a reference to that object but rather the
 blessed object reference itself. (Otherwise, you'd get a reference
 to that blessed object).
 
+=head1 MEMORY STORE
+
+The Storable engine can also store data into a Perl scalar instead, to
+later retrieve them. This is mainly used to freeze a complex structure in
+some safe compact memory place (where it can possibly be sent to another
+process via some IPC, since freezing the structure also serializes it in
+effect). Later on, and maybe somewhere else, you can thaw the Perl scalar
+out and recreate the original complex structure in memory.
+
+Surprisingly, the routines to be called are named C<freeze> and C<thaw>.
+If you wish to send out the frozen scalar to another machine, use
+C<nfreeze> instead to get a portable image.
+
+Note that freezing an object structure and immediately thawing it
+actually achieves a deep cloning of that structure. Storable provides
+you with a C<dclone> interface which does not create that intermediary
+scalar but instead freezes the structure in some internal memory space
+and then immediatly thaws it out.
+
 =head1 SPEED
 
 The heart of Storable is written in C for decent speed. Extra low-level
@@ -201,12 +278,37 @@ Storage is usually faster than retrieval since the latter has to
 allocate the objects from memory and perform the relevant I/Os, whilst
 the former mainly performs I/Os.
 
-On my HP 9000/712 machine running HPUX 9.03, I can store 1 Mbyte/s
-and I can retrieve at 0.86 Mbytes/s, approximatively
+On my HP 9000/712 machine running HPUX 9.03 and with perl 5.004, I can
+store 0.8 Mbyte/s and I can retrieve at 0.72 Mbytes/s, approximatively
 (CPU + system time).
 This was measured with Benchmark and the I<Magic: The Gathering>
 database from Tom Christiansen (1.9 Mbytes).
 
+=head1 EXAMPLES
+
+Here are some code samples showing a possible usage of Storable:
+
+	use Storable qw(store retrieve freeze thaw dclone);
+
+	%color = ('Blue' => 0.1, 'Red' => 0.8, 'Black' => 0, 'White' => 1);
+
+	store(\%color, '/tmp/colors') or die "Can't store %a in /tmp/colors!\n";
+
+	$colref = retrieve('/tmp/colors');
+	die "Unable to retrieve from /tmp/colors!\n" unless defined $colref;
+	printf "Blue is still %lf\n", $colref->{'Blue'};
+
+	$colref2 = dclone(\%color);
+
+	$str = freeze(\%color);
+	printf "Serialization of %%color is %d bytes long.\n", length($str);
+	$colref3 = thaw($str);
+
+which prints (on my machine):
+
+	Blue is still 0.100000
+	Serialization of %color is 102 bytes long.
+
 =head1 WARNING
 
 If you're using references as keys within your hash tables, you're bound
@@ -223,6 +325,9 @@ references, will probably differ from the original addresses. The
 topology of your structure is preserved, but not hidden semantics
 like those.
 
+On platforms where it matters, be sure to call C<binmode()> on the
+descriptors that you pass to Storable functions.
+
 =head1 BUGS
 
 You can't store GLOB, CODE, FORMLINE, etc... If you can define
